ASMB,A,L,B,T
      ORG 37107B
* 
*********** SYSTEM INPUT/OUTPUT DRIVER ********** 
* 
* THIS DRIVER PROVIDES THE PUNCH OUTPUT FACILITY
*  REQUIRED BY THE ASSEMBLER, SYMBOLIC EDITOR AND 
*  FORTRAN COMPILER. THE ADDRESS FOR THIS FACILITY
*  IS AT LOCATION 103 IN THE SYSTEM LINKAGE TABLE.
* 
* OUTPUT RECORDS ARE EITHER SYMBOLIC (ASCII MODE) 
*  OR BINARY (DIRECT TRANSFER). THE MODE OF THE 
*  OUTPUT, AND TYPE OF RECORD TO BE PRODUCED, IS
*  INDICATED BY THE PARAMETER IN THE A-REGISTER 
*  UPON ENTRY TO THE DRIVER ( THIS PARAMETER IS 
*  THE BUFFER LENGTH.) THE DRIVER PROVIDES A
*  RETURN/LINE-FEED AT THE END OF AN ASCII RECORD.
*  THE DRIVER PROVIDES FOUR (4) FEED-FRAMES AT
*  THE END OF A BINARY RECORD.
* 
* 
* CALLING SEQUENCE: 
* 
*      (A) = LENGTH (AND MODE) OF RECORD TO BE OUTPUT 
*      (B) = STARTING ADDRESS OF DATA BUFFER
* 
* (P)    JSB 103B,I   LOCATION 103 CONTAINS ADDRESS 
* (P+1)  (RETURN) 
* 
*  ASCII (SYMBOLIC) OUTPUT IS INDICATED BY A POSITIVE 
*   VALUE IN A- THIS VALUE IS THE NUMBER OF CHARACTERS
*   TO BE OUTPUT. 
* 
*  BINARY OUTPUT IS INDICATED BY A NEGATIVE (2S COMPLE- 
*   MENT) VALUE IN A.  THIS NEGATIVE VALUE IS THE 
*   NUMBER OF WODS TO BE OUTPUT.. 
* 
* 
*****  DRIVER INITIALIZATION SECTION  ***** 
* 
* PURPOSE OF THIS SECTION IS TO SET THE I/O 
*  INSTRUCTIONS TO REFERENCE A DESIGNATED I/O 
*  ADDRESS FOR THE 2753A TAPE PUNCH.
* 
* EXECUTION OF THIS SECTION IS BEGUN AT LOCATION 2: 
*      (2)  JMP  3,I
*      (3)  DEF  I.PTP
*  WITH SWITCHES 05-00 = I/O ADDRESS. 
* 
* THIS SECTION IS RELEASED AFTER THE INITIALIZATION 
*  IS COMPLETED.
* 
      ORG 2 
* 
      JMP 3,I      SET STARTING 
      DEF I.PTP     POINT IN 2 AND 3
      ORR 
* 
I.PTP LIA 01       GET AND
      AND M.77      ISOLATE I/O ADDRESS 
      STA 1        SET IN B.
      LDA PP08     -OR- 
      IOR 1         I/O 
      STA PP08      ADDRESS 
      LDA POUT+1    IN
      IOR 1         ALL 
      STA POUT+1    I/O 
      LDA POUT+2    INSTRUCTIONS. 
      IOR 1 
      STA POUT+2
      LDA POUT+3
      IOR 1 
      STA POUT+3
      LDA PP04
      IOR 1 
      STA PP04
* 
      CCA          (A) = 177777 
      HLT 77B       HALT, 
      JMP *-1        IRRECOVERABLE. 
* 
M.77  OCT 77
* 
LWAM  EQU M.77     DEFINE LWA AVAILABLE MEMORY
* 
PT.P NOP           ENTRY/EXIT 
     STB BUFF      SAVE STARTING ADDRESS OF BUFFER
     CCE,SSA       IF ASCII MODE, SET E=1, AND SKIP 
     ALS,CLE,SLA   IF BINARY, SET E=0, CONVERT WORDS
*                   TO CHARS AND FORCE A SKIP 
PP01 CMA,INA       SET NEGATIVE ASCII COUNT AND SAVE
      SZA,RSS      IF BUFFER LENGTH ZERO, GO TO 
      JMP PP05      OUTPUT A LINE-FEED. 
     STA CHCT      AS COUNTER FOR OUTPUT SECTION. 
     LDB ULFL      SET UPPER/LOWER CHARACTER INDICA-
*                   TOR TO UPPER (SIGN - 0).
* 
* CHARACTER OUTPUT SECTION
* 
PP02 LDA BUFF,I    GET WORD CONTAINING CHARACTER
     SSB,RSS       IF UPPER/LOWER FLAG SAYS UPPER 
     ALF,ALF        (SIGN = 0) ROTATE TO LOWER
     AND M377      REMOVE UPPER CHARACTER 
     SEZ            SET 8-LEVEL BIT (BIT 07) = 1
     IOR M200        IF ASCII MODE
     JSB POUT      OUTPUT CHAR. IN A(07-00).
     SSB           IF CHAR. OUTPUT WAS LOWER CHAR., 
     ISZ BUFF       ADD 1 TO BUFFER ADDRESS.
     RBL           SET UPPER/LOWER FLAG FOR NEXT CHAR 
     ISZ CHCT      INDEX CHARACTER COUNTER. 
     JMP PP02       NOT ZERO, MORE TO OUTPUT. 
* 
*  COMPLETION SECTION 
* 
     SEZ,RSS      IF BINARY MODE (E=0), JMMP. 
     JMP PP03       *ASCII MODE SECTION*
     LDA RETN      SET A = RETURN CODE
     JSB POUT        AND OUTPUT RETURN
PP05 LDA LINF      SET A = LINE-FEED CODE 
     JSB POUT        AND OUTPUT LINE-FEED 
     JMP PP04      GO TO EXIT SECTION 
* 
PP03 LDA CM04      BINARY SECTION.  SET BUFF = -4 
     STA BUFF       FOR COUNTER FOR FEED-FRAMES 
     CLA           SET A = ZERO( FEED-FRAME). 
     JSB POUT      OUTPUT FEED-FRAME
     ISZ BUFF      INDEX COUNTER
     JMP *-3        CONTINUE OUTPUT 
* 
*  EXIT SECTION.  RECORD OUTPUT COMPLETED.
* 
PP04  LIA IOAX     GET DEVICE STATUS. 
      SZA,RSS      IF NOT TAPE SUPPLY LOW,
      JMP PP08      GO TO EXIT. 
* 
      LDB CM100   *TAPE SUPPLY LOW* 
PP06  CLA          OUTPUT 
      JSB  POUT      10 INCHES
      INB,SZB        OF 
      JMP PP06        TRAILER 
* 
      HLT 66B      TAPE SUPPLY LOW HALT 
* 
      LDB CM100    ON RESTART, OUTPUT 
PP07  CLA           10 INCHES 
      JSB POUT       FOR
      INB,SZB         LEADER
      JMP PP07
* 
PP08  CLC IOAX     CLEAR DEVICE CONTROL 
     JMP PT.P,I     AND EXIT TO CALLER
* 
*  SUBROUTINE  (POUT) TO OUTPUT ONE CHARACTER TO
*   PUNCH DEVICE. 
*  CALLING SEQUENCE:
*             (A) = CHARACTER IN BITS 07-00 
*  (P)        JSB  POUT 
*  (P+1)      (RETURN)    ( (B) NOT ALTERED ) 
* 
POUT NOP      ENTRY/EXIT POINT
     OTA IOAX      OUTPUT CHAR. TO  I/O REGISTER
     STC IOAX,C    TELL DEVICE TO PUNCH - CLEAR FLAG
     SFS IOAX      DELAY UNTIL CHARACTER
     JMP *-1        IS PUNCHED. 
     JMP POUT,I     EXIT TO CALLING POINT.
* 
* CONSTANT  AND  STORAGE  SECTION 
* 
BUFF OCT 0         HOLDS CURRENT BUFFER ADDRESS 
CHCT OCT 0         HOLDS INDEX FOR CHARACTER COUNT
ULFL OCT 52525     UPPER/LOWER CHARACTER POSITION 
*                     INDICATOR. 0=UPPER, 1=LOWER.
M377 OCT 377       MASK FOR  8-BIT CHARACTER
M200 OCT 200       CONSTANT FOR SETTING 8-LEVEL BIT 
RETN OCT 215       RETURN CODE (WITH 8-LEVEL SET) 
LINF OCT 212       LINE FEED CODE (WITH 8-LEVEL SET)
CM04 OCT 177774    MINUS 4 - USED FOR COUNTER 
CM100 DEC -100     COUNTER FOR LEADER/TRAILER 
* 
* FOLLOWING STATEMENTS SET  ADDRESS OF DRIVER 
*     INTO SYSTEM LINKAGE TABLE ( LOC. 103 )
* 
     ORG 103B 
     ABS PT.P 
* 
     ORG 106B 
     ABS LWAM      SET LWA AVAILABLE MEMORY 
* 
IOAX  EQU 0 
* 
     END
