#!/bin/bash
#
# this script assembles HP21xx assembly code using vintage assembler
# requires xterm hp2100 sio.abs extasmb.abs
# multipart asm requires blassic flip
#
# edit below to specify directory containing sio.abs and extasmb.abs
bindir=~/hpbin
# specify directory for temp files
tmpdir=/tmp

function WriteScript {
 echo  > $tmpdir/extasmb.tmp "set cpu 21MX"
 echo >> $tmpdir/extasmb.tmp "set cpu 256K"
 echo >> $tmpdir/extasmb.tmp "set clk dev=10"
 echo >> $tmpdir/extasmb.tmp "set tty dev=11"
 echo >> $tmpdir/extasmb.tmp "set ptr dev=12"
 echo >> $tmpdir/extasmb.tmp "set ptp dev=13"
 echo >> $tmpdir/extasmb.tmp "attach ptp $base.$outext"
 echo >> $tmpdir/extasmb.tmp "load $bindir/sio.abs"
 echo >> $tmpdir/extasmb.tmp "load $bindir/extasmb.abs"
 if [ $asmbcount -gt 1 ]; then
  echo >> $tmpdir/extasmb.tmp "d s 1"
 fi
 while [ "$1" != "" ]; do
  echo >> $tmpdir/extasmb.tmp "attach ptr $1"
  echo >> $tmpdir/extasmb.tmp "run 100"
  echo >> $tmpdir/extasmb.tmp "attach ptr $1"
  echo >> $tmpdir/extasmb.tmp "c"
  shift
 done
 echo >> $tmpdir/extasmb.tmp "detach ptp"
 echo >> $tmpdir/extasmb.tmp "exit"
}

if [ -e $1 ]; then
 outext=none
 base=`echo -n $1 | head -c -4`
 grep "^ASMB," $1 | grep -q ",A" && outext=abs
 grep "^ASMB," $1 | grep -q ",R" && outext=rel
 if [ "$outext" == "none" ]; then
  echo "missing ASMB line"
 else
  asmbcount=`grep -c "^ASMB," $1`
  if [ $asmbcount -ne 1 ]; then
   # need to split up files.. somehow... must keep in msdos format
   # this solution requires flip and blassic, it's a mess but works
   echo "splitting up multipart source"
   mkdir $tmpdir/extasmb_m
   cp $1 $tmpdir/extasmb.f
   flip -u $tmpdir/extasmb.f
   echo  > $tmpdir/extasmb.bas "100 open \"$tmpdir/extasmb.f\" for input as #1"
   echo >> $tmpdir/extasmb.bas "110 count = 1 : print count"
   echo >> $tmpdir/extasmb.bas "120 open \"$tmpdir/extasmb_m/\"+str\$(count)+\".asm\" for output as #2"
   echo >> $tmpdir/extasmb.bas "130 line input #1,a\$ : print #2,a\$"
   echo >> $tmpdir/extasmb.bas "140 line input #1,a\$ : print #2,a\$"
   echo >> $tmpdir/extasmb.bas "150 while not eof(1)"
   echo >> $tmpdir/extasmb.bas "160 line input #1,a\$ : if left\$(a\$,5)<>\"ASMB,\" then 190"
   echo >> $tmpdir/extasmb.bas "170 close #2 : count = count + 1 : print count"
   echo >> $tmpdir/extasmb.bas "180 open \"$tmpdir/extasmb_m/\"+str\$(count)+\".asm\" for output as #2"
   echo >> $tmpdir/extasmb.bas "190 print #2,a\$"
   echo >> $tmpdir/extasmb.bas "200 wend : close #2"
   xterm -e blassic $tmpdir/extasmb.bas
   rm $tmpdir/extasmb.f
   rm $tmpdir/extasmb.bas
   flip -m $tmpdir/extasmb_m/*.asm
   WriteScript $tmpdir/extasmb_m/*.asm
  else
   WriteScript $1
  fi
  echo "assembling $outext..."
  xterm -e "hp2100 $tmpdir/extasmb.tmp > $base.lst"
  rm $tmpdir/extasmb.tmp
  if [ -e $tmpdir/extasmb_m ]; then
   rm -r $tmpdir/extasmb_m
  fi
 fi
else
 echo "source not found"
fi

